#include "EntitySnapshot.h"

#include <ArmarXCore/core/exceptions/local/ExpressionException.h>

#include "error.h"


namespace armarx::armem::d_ltm
{

    std::filesystem::path EntitySnapshot::_fullPath() const
    {
        if (path)
        {
            return _fullPath(*path);
        }
        return std::filesystem::path();
    }

    std::filesystem::path EntitySnapshot::_fullPath(const std::filesystem::path& path) const
    {
        return path / id().memoryName / id().coreSegmentName / id().providerSegmentName
               / id().entityName
               / std::to_string(id().timestamp.toMicroSeconds());
    }

    wm::EntitySnapshot EntitySnapshot::convert(const aron::typenavigator::NavigatorPtr& expectedStructure) const
    {
        wm::EntitySnapshot m;
        for (const auto& s : _container)
        {
            m.addInstance(s.convert(expectedStructure));
        }
        return m;
    }

    void EntitySnapshot::reload(const std::shared_ptr<std::filesystem::path>& p_ptr)
    {
        if (!p_ptr)
        {
            ARMARX_WARNING << "The entered is NULL.";
        }
        std::filesystem::path p = _fullPath(*p_ptr);
        if (!std::filesystem::is_directory(p))
        {
            ARMARX_ERROR << "The entered path is not leading to a directory! Every EntitySnapshot must at least contain one EntityInstance.";
        }
        else
        {
            _container.clear();
            path = p_ptr;

            // todo
            for (int i = 0; i < 1000; ++i)
            {
                std::filesystem::path d = p / std::to_string(i);
                if (std::filesystem::is_directory(d))
                {
                    auto wms = _container.emplace_back(id().withInstanceIndex(i));
                    wms.reload(p_ptr);
                }
                else
                {
                    break;
                }
            }
        }
    }

    void EntitySnapshot::setTo(const wm::EntitySnapshot& m)
    {
        try
        {
            std::filesystem::create_directories(_fullPath());
        }
        catch (...)
        {
            ARMARX_WARNING << GetHandledExceptionString();
            return;
        }

        // We remove the contente here and reset it with new values
        _container.clear();

        int i = 0;
        for (const auto& s : m.instances())
        {
            try
            {
                std::filesystem::create_directory(_fullPath() / std::to_string(i));
            }
            catch (...)
            {
                ARMARX_WARNING << GetHandledExceptionString();
                continue;;
            }

            auto wms = _container.emplace_back(id().withInstanceIndex(i++));
            wms.path = path;
            wms.setTo(s);
        }
    }
}
