#pragma once

#include "../base/EntityInstanceBase.h"

#include "../workingmemory/EntityInstance.h"
#include "mongodb/MongoDBConnectionManager.h"

#include "../workingmemory/entityInstance_conversions.h"
#include "../workingmemory/json_conversions.h"

namespace armarx::armem::ltm
{

    /**
     * @brief Metadata of an entity instance.
     */
    struct EntityInstanceMetadata
    {
        /// Time when this value was created.
        Time timeCreated;
        /// Time when this value was sent to the memory.
        Time timeSent;
        /// Time when this value has arrived at the memory.
        Time timeArrived;

        /// An optional confidence, may be used for things like decay.
        float confidence = 1.0;


        bool operator==(const EntityInstanceMetadata& other) const;
        inline bool operator!=(const EntityInstanceMetadata& other) const
        {
            return !(*this == other);
        }
    };

    /**
     * @brief Data of a single entity instance.
     */
    class EntityInstance :
        public base::EntityInstanceBase<EntityInstance>
    {
        using Base = base::EntityInstanceBase<EntityInstance>;

    public:

        using Base::EntityInstanceBase;

        EntityInstance(const EntityInstance& other) = default;
        EntityInstance(EntityInstance&& other) = default;
        EntityInstance& operator=(const EntityInstance& other) = default;
        EntityInstance& operator=(EntityInstance&& other) = default;


        EntityInstanceMetadata& metadata()
        {
            return _metadata;
        }
        inline const EntityInstanceMetadata& metadata() const
        {
            return _metadata;
        }


        /**
         * @brief Fill `*this` with the update's values.
         * @param update The update.
         * @param index The instances index.
         */
        virtual void update(const EntityUpdate& update, int index) override;

        virtual bool equalsDeep(const EntityInstance& other) const override;

        virtual EntityInstance copy() const override;

    protected:
        virtual void _copySelf(EntityInstance& other) const override;


    private:
        /// The metadata.
        EntityInstanceMetadata _metadata;
    };
}
