/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

// STD/STL
#include <memory>
#include <map>
#include <vector>
#include <stack>

// Base Class
#include <RobotAPI/libraries/aron/core/Factory.h>

// ArmarX
#include <ArmarXCore/core/rapidxml/wrapper/RapidXmlReader.h>
#include <RobotAPI/libraries/aron/core/codegenerator/GenerateTypeInfo.h>
#include <RobotAPI/libraries/aron/core/codegenerator/GenerateIntEnumInfo.h>
#include <RobotAPI/libraries/aron/core/navigator/type/AllNavigators.h>


namespace armarx::aron::xmltypereader
{
    class ReaderFactory;
    typedef std::shared_ptr<ReaderFactory> ReaderFactoryPtr;

    class ReaderFactory :
        virtual public aron::Factory<RapidXmlReaderNode, typenavigator::NavigatorPtr>
    {
    public:
        ReaderFactory() = default;

        virtual typenavigator::NavigatorPtr create(const RapidXmlReaderNode&, const Path&) const override;
        virtual typenavigator::NavigatorPtr createSpecific(const RapidXmlReaderNode&, const Path&) const override;

    protected:
        static typenavigator::NavigatorPtr ResolveTypename(const std::string&);
        static type::Maybe GetMaybe(const RapidXmlReaderNode&);

    public:
        static std::map<std::string, codegeneratorhelper::GenerateObjectInfoPtr> AllPublicObjects;
        static std::map<std::string, codegeneratorhelper::GenerateIntEnumInfoPtr> AllPublicIntEnums;

    protected:
        static std::stack<codegeneratorhelper::GenerateObjectInfoPtr> ObjectGenerationStack;

    };

#define RUN_ARON_MACRO(upperType, lowerType, capsType) \
    class upperType##ReaderFactory : \
        virtual public ReaderFactory \
    { \
    public: \
        upperType##ReaderFactory() = default; \
        virtual typenavigator::NavigatorPtr createSpecific(const RapidXmlReaderNode&, const Path&) const override; \
    };

    HANDLE_ALL_ARON_TYPES
#undef RUN_ARON_MACRO
}
