/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

// STD/STL
#include <memory>
#include <unordered_map>

// BaseClass
#include <RobotAPI/libraries/aron/core/navigator/NavigatorFactory.h>

// ArmarX
#include <RobotAPI/libraries/aron/core/navigator/data/Navigator.h>
#include <RobotAPI/libraries/aron/core/Descriptor.h>

namespace armarx::aron::datanavigator
{
    class NavigatorFactory;
    typedef std::shared_ptr<NavigatorFactory> NavigatorFactoryPtr;

    class NavigatorFactory :
        virtual public armarx::aron::NavigatorFactory<data::AronDataPtr, datanavigator::NavigatorPtr, data::Descriptor>
    {
    public:
        NavigatorFactory() = default;
        virtual NavigatorPtr create(const data::AronDataPtr&, const Path&) const override;
        virtual NavigatorPtr createSpecific(const data::AronDataPtr&, const Path&) const override;
    };

    // Factories
#define RUN_ARON_MACRO(upperType, lowerType, capsType) \
    class upperType##NavigatorFactory : \
        virtual public NavigatorFactory \
    { \
    public: \
        upperType##NavigatorFactory() = default; \
        virtual NavigatorPtr createSpecific(const data::AronDataPtr&, const Path&) const override; \
    };

    HANDLE_ALL_ARON_DATA
#undef RUN_ARON_MACRO

}
