/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

// STD/STL
#include <memory>

// Base class
#include "../detail/PrimitiveNavigatorBase.h"

// ArmarX
#include "../../type/primitive/Double.h"

namespace armarx::aron::datanavigator
{

    class DoubleNavigator;
    typedef std::shared_ptr<DoubleNavigator> DoubleNavigatorPtr;

    class DoubleNavigator :
        virtual public detail::PrimitiveNavigatorBase<data::AronDouble, double, DoubleNavigator>
    {
    public:
        /* constructors */
        DoubleNavigator(const Path& = Path());
        DoubleNavigator(const data::AronDoublePtr&, const Path& = Path());
        DoubleNavigator(const double, const Path& = Path());

        /* operators */
        bool operator==(const DoubleNavigator&) const override;
        bool operator==(const DoubleNavigatorPtr&) const override;

        /* static methods */
        static DoubleNavigatorPtr FromAronDoublePtr(const data::AronDoublePtr& aron);
        static data::AronDoublePtr ToAronDoublePtr(const DoubleNavigatorPtr& navigator);

        /* public member functions */
        data::AronDoublePtr toAronDoublePtr() const;

        /* virtual implementations */
        virtual std::string getName() const override;

        virtual typenavigator::NavigatorPtr recalculateType() const override;
        virtual bool fullfillsType(const typenavigator::NavigatorPtr&) const override;
    };
}
