/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

// STD/STL
#include <string>
#include <map>

// Base class
#include "../detail/NavigatorBase.h"

namespace armarx::aron::typenavigator
{
    class IntEnumNavigator;
    typedef std::shared_ptr<IntEnumNavigator> IntEnumNavigatorPtr;

    class IntEnumNavigator :
        virtual public detail::NavigatorBase<type::AronIntEnum, IntEnumNavigator>
    {
    public:
        // constructors
        IntEnumNavigator(const Path& path = Path());
        IntEnumNavigator(const type::AronIntEnumPtr&, const Path& path = Path());

        // operators
        virtual bool operator==(const IntEnumNavigator&) const override;

        // public member functions
        std::map<std::string, int> getAcceptedValueMap() const;
        std::vector<std::string> getAcceptedValueNames() const;
        std::vector<int> getAcceptedValues() const;

        std::string getValueName(int) const;
        int getValue(const std::string&) const;
        std::string getEnumName() const;

        void setEnumName(const std::string&);
        void addAcceptedValue(const std::string&, int);

        type::AronIntEnumPtr toAronIntEnumPtr() const;

        // virtual implementations
        virtual std::string getName() const override;
        virtual std::vector<NavigatorPtr> getChildren() const override;
        virtual size_t childrenSize() const override;
    };
}
