/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

// Header
#include "EigenQuaternion.h"

#include <SimoxUtility/algorithm/string/string_conversion.h>

namespace armarx::aron::typenavigator
{
    // constructors
    EigenQuaternionNavigator::EigenQuaternionNavigator(const Path& path) :
        aron::Navigator<type::Descriptor, type::AronType>::Navigator(type::Descriptor::eEigenQuaternion, path)
    {
    }

    EigenQuaternionNavigator::EigenQuaternionNavigator(const type::AronEigenQuaternionPtr& o, const Path& path) :
        aron::Navigator<type::Descriptor, type::AronType>::Navigator(type::Descriptor::eEigenQuaternion, path),
        detail::NavigatorBase<type::AronEigenQuaternion, EigenQuaternionNavigator>(o)
    {
    }

    // operators
    bool EigenQuaternionNavigator::operator==(const EigenQuaternionNavigator& other) const
    {
        if (getMaybe() != other.getMaybe())
        {
            return false;
        }
        return getTypename() == other.getTypename();
    }

    std::string EigenQuaternionNavigator::checkTypename(const std::string& s) const
    {
        for (const auto& [k, t] : ACCEPTED_TYPES)
        {
            if (s == k || std::find(t.begin(), t.end(), s) != t.end())
            {
                return k;
            }
        }
        throw error::StringNotValidException("EigenQuaternionNavigator", "checkTypename", "The typename is wrong.", s, getPath());
    }

    type::AronEigenQuaternionPtr EigenQuaternionNavigator::toAronEigenQuaternionPtr() const
    {
        return this->aron;
    }

    std::string EigenQuaternionNavigator::getTypename() const
    {
        return this->aron->typeName;
    }

    void EigenQuaternionNavigator::setTypename(const std::string& u)
    {
        this->aron->typeName = checkTypename(u);
    }

    // virtual implementations
    std::string EigenQuaternionNavigator::getName() const
    {
        return "AronEigenQuaternionType<" + simox::alg::to_string(ACCEPTED_DIMENSION, ", ") + ", " + this->aron->typeName + ">";
    }
}

