/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

// STD/STL
#include <string>
#include <map>

// Base class
#include "../detail/NDArrayNavigatorBase.h"

namespace armarx::aron::typenavigator
{
    class IVTCByteImageNavigator;
    typedef std::shared_ptr<IVTCByteImageNavigator> IVTCByteImageNavigatorPtr;

    class IVTCByteImageNavigator :
        virtual public detail::NDArrayNavigatorBase<type::AronIVTCByteImage, IVTCByteImageNavigator>
    {
    public:
        // constructors
        IVTCByteImageNavigator(const Path& path = Path());
        IVTCByteImageNavigator(const type::AronIVTCByteImagePtr&, const Path& path);

        // operators
        virtual bool operator==(const IVTCByteImageNavigator&) const override;

        // public member functions
        type::AronIVTCByteImagePtr toAronIVTCByteImagePtr() const;

        // virtual implementations
        virtual std::string getName() const override;

    public:
        const std::map<std::string, std::vector<std::string>> ACCEPTED_TYPES =
        {
            {"GrayScale", {"CByteImage::ImageType::eGrayScale", "ImageType::eGrayScale"}},
            {"RGB24", {"CByteImage::ImageType::eRGB24", "ImageType::eRGB24"}},
            {"RGB24Split", {"CByteImage::ImageType::eRGB24Split", "ImageType::eRGB24Split"}}
        };
    };
}
